@@ -273,8 +273,8 @@ gnc_customer_window_ok_cb (GtkWidget *widget, gpointer data)
     if (check_entry_nonempty (cw->dialog, cw->company_entry,
                               _("You must enter a company name. "
                                 "If this customer is an individual (and not a company) "
-                                "you should set the \"company name\" and \"contact name\" "
-                                "the same.")))
+                                "you should enter the same value for:\nIdentification "
+                                "- Company Name, and\nPayment Address - Name.")))
         return;
 
     /* Make sure we have an address */
@@ -305,8 +305,7 @@ gnc_customer_window_ok_cb (GtkWidget *widget, gpointer data)
     /* Set the customer id if one has not been chosen */
     if (safe_strcmp (gtk_entry_get_text (GTK_ENTRY (cw->id_entry)), "") == 0)
     {
-        string = g_strdup_printf ("%.6" G_GINT64_FORMAT,
-                                  gncCustomerNextID (cw->book));
+        string = gncCustomerNextID (cw->book);
         gtk_entry_set_text (GTK_ENTRY (cw->id_entry), string);
         g_free(string);
     }
