@@ -314,6 +314,65 @@ GncInvoice *gncInvoiceCreate (QofBook *book)
     return invoice;
 }
 
+GncInvoice *gncInvoiceCopy (const GncInvoice *from)
+{
+    GncInvoice *invoice;
+    QofBook* book;
+    GList *node;
+
+    g_assert(from);
+    book = qof_instance_get_book(from);
+    g_assert(book);
+
+    invoice = g_object_new (GNC_TYPE_INVOICE, NULL);
+    qof_instance_init_data (&invoice->inst, _GNC_MOD_NAME, book);
+
+    gncInvoiceBeginEdit(invoice);
+
+    invoice->id = CACHE_INSERT (from->id);
+    invoice->notes = CACHE_INSERT (from->notes);
+    invoice->billing_id = CACHE_INSERT (from->billing_id);
+    invoice->active = from->active;
+
+    invoice->terms = from->terms;
+    gncBillTermIncRef (invoice->terms);
+
+    gncOwnerCopy(&from->billto, &invoice->billto);
+    gncOwnerCopy(&from->owner, &invoice->owner);
+    invoice->job = from->job; // FIXME: Need IncRef or similar here?!?
+
+    invoice->to_charge_amount = from->to_charge_amount;
+    invoice->date_opened = from->date_opened;
+
+    // Copy all invoice->entries
+    for (node = from->entries; node; node = node->next)
+    {
+        GncEntry *from_entry = node->data;
+        GncEntry *to_entry = gncEntryCreate(book);
+        gncEntryCopy(from_entry, to_entry);
+
+        if (gncInvoiceGetOwnerType (invoice) == GNC_OWNER_VENDOR)
+        {
+            // this is a vendor bill
+            gncBillAddEntry(invoice, to_entry);
+        }
+        else
+        {
+            // this is an invoice
+            gncInvoiceAddEntry(invoice, to_entry);
+        }
+    }
+
+    // FIXME: The prices are not (yet) copied; is this a problem?
+
+    // Posted-date and the posted Txn is intentionally not copied; the
+    // copy isn't "posted" but needs to be posted by the user.
+
+    gncInvoiceCommitEdit(invoice);
+
+    return invoice;
+}
+
 void gncInvoiceDestroy (GncInvoice *invoice)
 {
     if (!invoice) return;
@@ -594,6 +653,8 @@ void gncInvoiceAddEntry (GncInvoice *invoice, GncEntry *entry)
 {
     GncInvoice *old;
 
+    g_assert(invoice);
+    g_assert(entry);
     if (!invoice || !entry) return;
 
     old = gncEntryGetInvoice (entry);
@@ -635,6 +696,8 @@ void gncBillAddEntry (GncInvoice *bill, GncEntry *entry)
 {
     GncInvoice *old;
 
+    g_assert(bill);
+    g_assert(entry);
     if (!bill || !entry) return;
 
     old = gncEntryGetBill (entry);
@@ -2014,6 +2077,24 @@ static const char * _gncInvoicePrintable (gpointer obj)
     return invoice->printname;
 }
 
+static void
+destroy_invoice_on_book_close(QofInstance *ent, gpointer data)
+{
+    GncInvoice* invoice = GNC_INVOICE(ent);
+
+    gncInvoiceBeginEdit(invoice);
+    gncInvoiceDestroy(invoice);
+}
+
+static void
+gnc_invoice_book_end(QofBook* book)
+{
+    QofCollection *col;
+
+    col = qof_book_get_collection(book, GNC_ID_INVOICE);
+    qof_collection_foreach(col, destroy_invoice_on_book_close, NULL);
+}
+
 static QofObject gncInvoiceDesc =
 {
     DI(.interface_version = ) QOF_OBJECT_VERSION,
@@ -2021,7 +2102,7 @@ static QofObject gncInvoiceDesc =
     DI(.type_label        = ) "Invoice",
     DI(.create            = ) (gpointer)gncInvoiceCreate,
     DI(.book_begin        = ) NULL,
-    DI(.book_end          = ) NULL,
+    DI(.book_end          = ) gnc_invoice_book_end,
     DI(.is_dirty          = ) qof_collection_is_dirty,
     DI(.mark_clean        = ) qof_collection_mark_clean,
     DI(.foreach           = ) qof_collection_foreach,
@@ -2107,22 +2188,22 @@ gboolean gncInvoiceRegister (void)
     return qof_object_register (&gncInvoiceDesc);
 }
 
-gint64 gncInvoiceNextID (QofBook *book, GncOwner *owner)
+gchar *gncInvoiceNextID (QofBook *book, GncOwner *owner)
 {
-    gint64 nextID;
+    gchar *nextID;
     switch (gncOwnerGetType(gncOwnerGetEndOwner(owner)))
     {
     case GNC_OWNER_CUSTOMER:
-        nextID = qof_book_get_counter (book, "gncInvoice");
+        nextID = qof_book_increment_and_format_counter (book, "gncInvoice");
         break;
     case GNC_OWNER_VENDOR:
-        nextID = qof_book_get_counter (book, "gncBill");
+        nextID = qof_book_increment_and_format_counter (book, "gncBill");
         break;
     case GNC_OWNER_EMPLOYEE:
-        nextID = qof_book_get_counter (book, "gncExpVoucher");
+        nextID = qof_book_increment_and_format_counter (book, "gncExpVoucher");
         break;
     default:
-        nextID = qof_book_get_counter (book, _GNC_MOD_NAME);
+        nextID = qof_book_increment_and_format_counter (book, _GNC_MOD_NAME);
         break;
     }
     return nextID;
