@@ -34,6 +34,7 @@
 #include "gnc-engine.h"
 #include "Account.h"
 #include "gnc-file.h"
+#include "gnc-filepath-utils.h"
 #include "gnc-gui-query.h"
 #include "gnc-hooks.h"
 #include "gnc-keyring.h"
@@ -398,14 +399,31 @@ show_session_error (QofBackendError io_error,
         gnc_error_dialog (parent, fmt, displayname);
         break;
 
+    case ERR_FILEIO_RESERVED_WRITE:
+        /* Translators: the first %s is a path in the filesystem,
+         * the second %s is PACKAGE_NAME, which by default is "GnuCash"
+         */
+        fmt = _("You attempted to save in\n%s\nor a subdirectory thereof.  "
+                "This is not allowed as %s reserves that directory for internal use.\n\n"
+                "Please try again in a different directory.");
+        gnc_error_dialog (parent, fmt, gnc_dotgnucash_dir(), PACKAGE_NAME);
+        break;
+
     case ERR_SQL_DB_TOO_OLD:
         fmt = _("This database is from an older version of GnuCash. "
-                "Do you want to want to upgrade the database "
-                "to the current version?");
-        if (gnc_verify_dialog (parent, TRUE, "%s", fmt))
-        {
-            uh_oh = FALSE;
-        }
+                "Select OK to upgrade it to the current version, Cancel "
+                "to mark it read-only.");
+
+        response = gnc_ok_cancel_dialog(parent, GTK_RESPONSE_CANCEL, "%s", fmt);
+        uh_oh = (response == GTK_RESPONSE_CANCEL);
+        break;
+
+    case ERR_SQL_DB_TOO_NEW:
+        fmt = _("This database is from a newer version of GnuCash. "
+                "This version can read it, but cannot safely save to it. "
+                "It will be marked read-only until you do File>Save As.");
+        gnc_warning_dialog (parent, "%s", fmt);
+        uh_oh = TRUE;
         break;
 
     case ERR_SQL_DB_BUSY:
@@ -417,6 +435,18 @@ show_session_error (QofBackendError io_error,
         gnc_error_dialog (parent, "%s", fmt);
         break;
 
+    case ERR_SQL_BAD_DBI:
+
+        fmt = _("The library \"libdbi\" installed on your system doesn't correctly "
+                "store large numbers.  This means GnuCash cannot use SQL databases "
+                "correctly.  Gnucash will not open or save to SQL databases until this is "
+                "fixed by installing a different version of \"libdbi\".  Please see "
+                "https://bugzilla.gnome.org/show_bug.cgi?id=611936 for more "
+                "information.");
+
+        gnc_error_dialog (parent, "%s", fmt);
+        break;
+
     default:
         PERR("FIXME: Unhandled error %d", io_error);
         fmt = _("An unknown I/O error (%d) occurred.");
@@ -720,8 +750,7 @@ gnc_post_file_open (const char * filename)
         }
     }
     /* if the database doesn't exist, ask the user ... */
-    else if ((ERR_BACKEND_NO_SUCH_DB == io_err) ||
-             (ERR_SQL_DB_TOO_OLD == io_err))
+    else if ((ERR_BACKEND_NO_SUCH_DB == io_err))
     {
         if (FALSE == show_session_error (io_err, newfile, GNC_FILE_DIALOG_OPEN))
         {
@@ -738,11 +767,11 @@ gnc_post_file_open (const char * filename)
     io_err = qof_session_get_error (new_session);
     if ((ERR_BACKEND_LOCKED == io_err) ||
             (ERR_BACKEND_READONLY == io_err) ||
-            (ERR_BACKEND_NO_SUCH_DB == io_err) ||
-            (ERR_SQL_DB_TOO_OLD == io_err))
+            (ERR_BACKEND_NO_SUCH_DB == io_err))
     {
         uh_oh = TRUE;
     }
+
     else
     {
         uh_oh = show_session_error (io_err, newfile, GNC_FILE_DIALOG_OPEN);
@@ -766,11 +795,10 @@ gnc_post_file_open (const char * filename)
         xaccLogEnable();
 
         /* check for i/o error, put up appropriate error dialog */
-        io_err = qof_session_get_error (new_session);
+        io_err = qof_session_pop_error (new_session);
 
         if (io_err == ERR_FILEIO_NO_ENCODING)
         {
-            qof_session_pop_error (new_session);
             if (gnc_xml_convert_single_file (newfile))
             {
                 /* try to load once again */
@@ -787,7 +815,24 @@ gnc_post_file_open (const char * filename)
         }
 
         uh_oh = show_session_error (io_err, newfile, GNC_FILE_DIALOG_OPEN);
-
+        /* Attempt to update the database if it's too old */
+        if ( !uh_oh && io_err == ERR_SQL_DB_TOO_OLD )
+        {
+            gnc_window_show_progress(_("Re-saving user data..."), 0.0);
+            qof_session_safe_save(new_session, gnc_window_show_progress);
+            io_err = qof_session_get_error(new_session);
+            uh_oh = show_session_error(io_err, newfile, GNC_FILE_DIALOG_SAVE);
+        }
+        /* Database is either too old and couldn't (or user didn't
+         * want it to) be updated or it's too new. Mark it as
+         * read-only
+         */
+        if (uh_oh && (io_err == ERR_SQL_DB_TOO_OLD ||
+                      io_err == ERR_SQL_DB_TOO_NEW))
+        {
+            qof_book_mark_readonly(qof_session_get_book(new_session));
+            uh_oh = FALSE;
+        }
         new_root = gnc_book_get_root_account (qof_session_get_book (new_session));
         if (uh_oh) new_root = NULL;
 
@@ -996,12 +1041,23 @@ gnc_file_do_export(const char * filename)
         newfile = norm_file;
     }
 
-    /* For file based uri's, remember the directory as the default. */
+    /* Some extra steps for file based uri's only */
     if (gnc_uri_is_file_protocol(protocol))
     {
+        /* Remember the directory as the default. */
         gchar *default_dir = g_path_get_dirname(path);
-        gnc_set_default_directory (GCONF_DIR_EXPORT, default_dir);
+        gnc_set_default_directory (GCONF_DIR_OPEN_SAVE, default_dir);
         g_free(default_dir);
+
+        /* Prevent user to store file in GnuCash' private configuration
+         * directory (~/.gnucash by default in linux, but can be overridden)
+         */
+        DEBUG("User path: %s, dotgnucash_dir: %s", path, gnc_dotgnucash_dir());
+        if (g_str_has_prefix(path, gnc_dotgnucash_dir()))
+        {
+            show_session_error (ERR_FILEIO_RESERVED_WRITE, newfile, GNC_FILE_DIALOG_SAVE);
+            return;
+        }
     }
 
     /* Check to see if the user specified the same file as the current
@@ -1210,12 +1266,23 @@ gnc_file_do_save_as (const char* filename)
         newfile = norm_file;
     }
 
-    /* For file based uri's, remember the directory as the default. */
+    /* Some extra steps for file based uri's only */
     if (gnc_uri_is_file_protocol(protocol))
     {
+        /* Remember the directory as the default. */
         gchar *default_dir = g_path_get_dirname(path);
         gnc_set_default_directory (GCONF_DIR_OPEN_SAVE, default_dir);
         g_free(default_dir);
+
+        /* Prevent user to store file in GnuCash' private configuration
+         * directory (~/.gnucash by default in linux, but can be overridden)
+         */
+        DEBUG("User path: %s, dotgnucash_dir: %s", path, gnc_dotgnucash_dir());
+        if (g_str_has_prefix(path, gnc_dotgnucash_dir()))
+        {
+            show_session_error (ERR_FILEIO_RESERVED_WRITE, newfile, GNC_FILE_DIALOG_SAVE);
+            return;
+        }
     }
 
     /* Check to see if the user specified the same file as the current
