@@ -33,7 +33,7 @@
 #include <gwenhywfar/gwenhywfar.h>
 #include <aqbanking/banking.h>
 
-#include "RecnWindow.h"
+#include "window-reconcile.h"
 #include "Transaction.h"
 #include "dialog-ab-trans.h"
 #include "gnc-ab-kvp.h"
@@ -410,30 +410,42 @@ gnc_ab_memo_to_gnc(const AB_TRANSACTION *ab_trans)
         AB_Transaction_GetRemoteAccountNumber(ab_trans);
     const gchar *ab_remote_bankcode =
         AB_Transaction_GetRemoteBankCode(ab_trans);
-    gchar *ab_other_accountid =
-        g_strdup(ab_remote_accountnumber ? ab_remote_accountnumber
-                 : _("unknown"));
-    gchar *ab_other_bankcode =
-        g_strdup(ab_remote_bankcode ? ab_remote_bankcode
-                 : _("unknown"));
+
+    gchar *ab_other_accountid = g_strdup(ab_remote_accountnumber);
+    gchar *ab_other_bankcode = g_strdup(ab_remote_bankcode);
+
+    gboolean have_accountid;
+    gboolean have_bankcode;
+
     gchar *retval;
 
-    g_strstrip(ab_other_accountid);
-    g_strstrip(ab_other_bankcode);
     /* Ensure string is in utf8 */
     gnc_utf8_strip_invalid(ab_other_accountid);
     gnc_utf8_strip_invalid(ab_other_bankcode);
 
-    if (ab_other_accountid && *ab_other_accountid)
+    /* and -then- trim it */
+    g_strstrip(ab_other_accountid);
+    g_strstrip(ab_other_bankcode);
+
+
+    have_accountid = ab_other_accountid && *ab_other_accountid;
+    have_bankcode = ab_other_bankcode && *ab_other_bankcode;
+
+    if ( have_accountid || have_bankcode )
     {
         retval = g_strdup_printf("%s %s %s %s",
-                                 _("Account"), ab_other_accountid,
-                                 _("Bank"), ab_other_bankcode);
+                                 have_accountid ? _("Account") : "",
+                                 have_accountid ? ab_other_accountid : "",
+                                 have_bankcode  ? _("Bank") : "",
+                                 have_bankcode  ? ab_other_bankcode : ""
+                                );
+        g_strstrip(retval);
     }
     else
     {
         retval = g_strdup("");
     }
+
     g_free(ab_other_accountid);
     g_free(ab_other_bankcode);
 
