@@ -215,7 +215,9 @@
 (define (qif-import:qif-to-gnc qif-files-list
                                qif-acct-map qif-cat-map
                                qif-memo-map stock-map
-                               default-currency-name progress-dialog)
+                               default-currency-name
+                               transaction-status-pref
+                               progress-dialog)
 
   ;; This procedure does all the work. We'll define it, then call it safely.
   (define (private-convert)
@@ -392,6 +394,7 @@
                                                  qif-acct-map
                                                  qif-cat-map
                                                  qif-memo-map
+                                                 transaction-status-pref
                                                  progress-dialog)
 
                   ;; rebalance and commit everything
@@ -423,6 +426,7 @@
 (define (qif-import:qif-xtn-to-gnc-xtn qif-xtn qif-file gnc-xtn
                                        gnc-acct-hash
                                        qif-acct-map qif-cat-map qif-memo-map
+                                       transaction-status-pref
                                        progress-dialog)
   (let ((splits (qif-xtn:splits qif-xtn))
         (gnc-near-split (xaccMallocSplit (gnc-get-current-book)))
@@ -490,10 +494,15 @@
 	      ;; the debit/credit lines. See bug 495219 for more information.
 	      (xaccTransSetNotes gnc-xtn qif-memo)))
 
+    ;; Look for the transaction status (QIF "C" line). When it exists, apply
+    ;; the cleared (c) or reconciled (y) status to the split. Otherwise, apply
+    ;; user preference.
     (if (eq? qif-cleared 'cleared)
-        (xaccSplitSetReconcile gnc-near-split #\c))
-    (if (eq? qif-cleared 'reconciled)
-        (xaccSplitSetReconcile gnc-near-split #\y))
+        (xaccSplitSetReconcile gnc-near-split #\c)
+        (if (eq? qif-cleared 'reconciled)
+            (xaccSplitSetReconcile gnc-near-split #\y)
+            ;; Apply user preference by default.
+            (xaccSplitSetReconcile gnc-near-split transaction-status-pref)))
 
     (if (not qif-security)
         (begin
